@@ -4,7 +4,7 @@
  *	  WAL replay logic for inverted index.
  *
  *
- * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -133,14 +133,6 @@ ginRedoInsertEntry(Buffer buffer, bool isLeaf, BlockNumber rightblkno, void *rda
 	}
 }
 
-/*
- * Redo recompression of posting list.  Doing all the changes in-place is not
- * always possible, because it might require more space than we've on the page.
- * Instead, once modification is required we copy unprocessed tail of the page
- * into separately allocated chunk of memory for further reading original
- * versions of segments.  Thanks to that we don't bother about moving page data
- * in-place.
- */
 static void
 ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 {
@@ -150,9 +142,6 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 	Pointer		segmentend;
 	char	   *walbuf;
 	int			totalsize;
-	Pointer		tailCopy = NULL;
-	Pointer		writePtr;
-	Pointer		segptr;
 
 	/*
 	 * If the page is in pre-9.4 format, convert to new format first.
@@ -162,37 +151,21 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 		ItemPointer uncompressed = (ItemPointer) GinDataPageGetData(page);
 		int			nuncompressed = GinPageGetOpaque(page)->maxoff;
 		int			npacked;
+		GinPostingList *plist;
 
-		/*
-		 * Empty leaf pages are deleted as part of vacuum, but leftmost and
-		 * rightmost pages are never deleted.  So, pg_upgrade'd from pre-9.4
-		 * instances might contain empty leaf pages, and we need to handle
-		 * them correctly.
-		 */
-		if (nuncompressed > 0)
-		{
-			GinPostingList *plist;
+		plist = ginCompressPostingList(uncompressed, nuncompressed,
+									   BLCKSZ, &npacked);
+		Assert(npacked == nuncompressed);
 
-			plist = ginCompressPostingList(uncompressed, nuncompressed,
-										   BLCKSZ, &npacked);
-			totalsize = SizeOfGinPostingList(plist);
-
-			Assert(npacked == nuncompressed);
-
-			memcpy(GinDataLeafPageGetPostingList(page), plist, totalsize);
-		}
-		else
-		{
-			totalsize = 0;
-		}
+		totalsize = SizeOfGinPostingList(plist);
 
+		memcpy(GinDataLeafPageGetPostingList(page), plist, totalsize);
 		GinDataPageSetDataSize(page, totalsize);
 		GinPageSetCompressed(page);
 		GinPageGetOpaque(page)->maxoff = InvalidOffsetNumber;
 	}
 
 	oldseg = GinDataLeafPageGetPostingList(page);
-	writePtr = (Pointer) oldseg;
 	segmentend = (Pointer) oldseg + GinDataLeafPageGetPostingListSize(page);
 	segno = 0;
 
@@ -210,6 +183,8 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 		ItemPointerData *newitems;
 		int			nnewitems;
 		int			segsize;
+		Pointer		segptr;
+		int			szleft;
 
 		/* Extract all the information we need from the WAL record */
 		if (a_action == GIN_SEGMENT_INSERT ||
@@ -232,17 +207,6 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 		Assert(segno <= a_segno);
 		while (segno < a_segno)
 		{
-			/*
-			 * Once modification is started and page tail is copied, we've
-			 * to copy unmodified segments.
-			 */
-			segsize = SizeOfGinPostingList(oldseg);
-			if (tailCopy)
-			{
-				Assert(writePtr + segsize < PageGetSpecialPointer(page));
-				memcpy(writePtr, (Pointer) oldseg, segsize);
-			}
-			writePtr += segsize;
 			oldseg = GinNextPostingListSegment(oldseg);
 			segno++;
 		}
@@ -283,42 +247,36 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 			Assert(a_action == GIN_SEGMENT_INSERT);
 			segsize = 0;
 		}
-
-		/*
-		 * We're about to start modification of the page.  So, copy tail of the
-		 * page if it's not done already.
-		 */
-		if (!tailCopy && segptr != segmentend)
-		{
-			int tailSize = segmentend - segptr;
-
-			tailCopy = (Pointer) palloc(tailSize);
-			memcpy(tailCopy, segptr, tailSize);
-			segptr = tailCopy;
-			oldseg = (GinPostingList *) segptr;
-			segmentend = segptr + tailSize;
-		}
+		szleft = segmentend - segptr;
 
 		switch (a_action)
 		{
 			case GIN_SEGMENT_DELETE:
-				segptr += segsize;
+				memmove(segptr, segptr + segsize, szleft - segsize);
+				segmentend -= segsize;
+
 				segno++;
 				break;
 
 			case GIN_SEGMENT_INSERT:
+				/* make room for the new segment */
+				memmove(segptr + newsegsize, segptr, szleft);
 				/* copy the new segment in place */
-				Assert(writePtr + newsegsize <= PageGetSpecialPointer(page));
-				memcpy(writePtr, newseg, newsegsize);
-				writePtr += newsegsize;
+				memcpy(segptr, newseg, newsegsize);
+				segmentend += newsegsize;
+				segptr += newsegsize;
 				break;
 
 			case GIN_SEGMENT_REPLACE:
-				/* copy the new version of segment in place */
-				Assert(writePtr + newsegsize <= PageGetSpecialPointer(page));
-				memcpy(writePtr, newseg, newsegsize);
-				writePtr += newsegsize;
-				segptr += segsize;
+				/* shift the segments that follow */
+				memmove(segptr + newsegsize,
+						segptr + segsize,
+						szleft - segsize);
+				/* copy the replacement segment in place */
+				memcpy(segptr, newseg, newsegsize);
+				segmentend -= segsize;
+				segmentend += newsegsize;
+				segptr += newsegsize;
 				segno++;
 				break;
 
@@ -328,18 +286,7 @@ ginRedoRecompress(Page page, ginxlogRecompressDataLeaf *data)
 		oldseg = (GinPostingList *) segptr;
 	}
 
-	/* Copy the rest of unmodified segments if any. */
-	segptr = (Pointer) oldseg;
-	if (segptr != segmentend && tailCopy)
-	{
-		int restSize = segmentend - segptr;
-
-		Assert(writePtr + restSize <= PageGetSpecialPointer(page));
-		memcpy(writePtr, segptr, restSize);
-		writePtr += restSize;
-	}
-
-	totalsize = writePtr - (Pointer) GinDataLeafPageGetPostingList(page);
+	totalsize = segmentend - (Pointer) GinDataLeafPageGetPostingList(page);
 	GinDataPageSetDataSize(page, totalsize);
 }
 
@@ -511,42 +458,11 @@ ginRedoDeletePage(XLogReaderState *record)
 	Buffer		lbuffer;
 	Page		page;
 
-	/*
-	 * Lock left page first in order to prevent possible deadlock with
-	 * ginStepRight().
-	 */
-	if (XLogReadBufferForRedo(record, 2, &lbuffer) == BLK_NEEDS_REDO)
-	{
-		page = BufferGetPage(lbuffer);
-		Assert(GinPageIsData(page));
-		GinPageGetOpaque(page)->rightlink = data->rightLink;
-		PageSetLSN(page, lsn);
-		MarkBufferDirty(lbuffer);
-	}
-
 	if (XLogReadBufferForRedo(record, 0, &dbuffer) == BLK_NEEDS_REDO)
 	{
 		page = BufferGetPage(dbuffer);
 		Assert(GinPageIsData(page));
 		GinPageGetOpaque(page)->flags = GIN_DELETED;
-
-		/*
-		 * deleteXid field of ginxlogDeletePage was added during backpatching.
-		 * But, non-backpatched instances will continue generate WAL without
-		 * this field.  We should be able to correctly apply that.  We can
-		 * distinguish new WAL records by size their data, because
-		 * ginxlogDeletePage changes its size on both 32-bit and 64-bit
-		 * platforms.
-		 */
-		StaticAssertStmt(sizeof(ginxlogDeletePage) !=
-						 sizeof(ginxlogDeletePageOld),
-						 "ginxlogDeletePage size should be changed "
-						 "with addition of deleteXid field");
-		Assert(XLogRecGetDataLen(record) == sizeof(ginxlogDeletePage) ||
-			   XLogRecGetDataLen(record) == sizeof(ginxlogDeletePageOld));
-		if (XLogRecGetDataLen(record) == sizeof(ginxlogDeletePage))
-			GinPageSetDeleteXid(page, data->deleteXid);
-
 		PageSetLSN(page, lsn);
 		MarkBufferDirty(dbuffer);
 	}
@@ -561,6 +477,15 @@ ginRedoDeletePage(XLogReaderState *record)
 		MarkBufferDirty(pbuffer);
 	}
 
+	if (XLogReadBufferForRedo(record, 2, &lbuffer) == BLK_NEEDS_REDO)
+	{
+		page = BufferGetPage(lbuffer);
+		Assert(GinPageIsData(page));
+		GinPageGetOpaque(page)->rightlink = data->rightLink;
+		PageSetLSN(page, lsn);
+		MarkBufferDirty(lbuffer);
+	}
+
 	if (BufferIsValid(lbuffer))
 		UnlockReleaseBuffer(lbuffer);
 	if (BufferIsValid(pbuffer))
@@ -824,13 +749,12 @@ gin_xlog_startup(void)
 {
 	opCtx = AllocSetContextCreate(CurrentMemoryContext,
 								  "GIN recovery temporary context",
-								  ALLOCSET_DEFAULT_MINSIZE,
-								  ALLOCSET_DEFAULT_INITSIZE,
-								  ALLOCSET_DEFAULT_MAXSIZE);
+								  ALLOCSET_DEFAULT_SIZES);
 }
 
 void
 gin_xlog_cleanup(void)
 {
 	MemoryContextDelete(opCtx);
+	opCtx = NULL;
 }
