@@ -3,7 +3,7 @@
  *
  *	controldata functions
  *
- *	Copyright (c) 2010-2015, PostgreSQL Global Development Group
+ *	Copyright (c) 2010-2016, PostgreSQL Global Development Group
  *	src/bin/pg_upgrade/controldata.c
  */
 
@@ -58,7 +58,6 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	bool		got_large_object = false;
 	bool		got_date_is_int = false;
 	bool		got_data_checksum_version = false;
-	bool		got_cluster_state = false;
 	char	   *lc_collate = NULL;
 	char	   *lc_ctype = NULL;
 	char	   *lc_monetary = NULL;
@@ -112,72 +111,6 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	pg_putenv("LC_ALL", NULL);
 	pg_putenv("LC_MESSAGES", "C");
 
-	/*
-	 * Check for clean shutdown
-	 */
-	if (!live_check || cluster == &new_cluster)
-	{
-		/* only pg_controldata outputs the cluster state */
-		snprintf(cmd, sizeof(cmd), "\"%s/pg_controldata\" \"%s\"",
-				 cluster->bindir, cluster->pgdata);
-		fflush(stdout);
-		fflush(stderr);
-
-		if ((output = popen(cmd, "r")) == NULL)
-			pg_fatal("could not get control data using %s: %s\n",
-					 cmd, strerror(errno));
-
-		/* we have the result of cmd in "output". so parse it line by line now */
-		while (fgets(bufin, sizeof(bufin), output))
-		{
-			if ((p = strstr(bufin, "Database cluster state:")) != NULL)
-			{
-				p = strchr(p, ':');
-
-				if (p == NULL || strlen(p) <= 1)
-					pg_fatal("%d: database cluster state problem\n", __LINE__);
-
-				p++;				/* remove ':' char */
-
-				/*
-				 * We checked earlier for a postmaster lock file, and if we found
-				 * one, we tried to start/stop the server to replay the WAL.  However,
-				 * pg_ctl -m immediate doesn't leave a lock file, but does require
-				 * WAL replay, so we check here that the server was shut down cleanly,
-				 * from the controldata perspective.
-				 */
-				/* remove leading spaces */
-				while (*p == ' ')
-					p++;
-				if (strcmp(p, "shut down in recovery\n") == 0)
-				{
-					if (cluster == &old_cluster)
-						pg_fatal("The source cluster was shut down while in recovery mode.  To upgrade, use \"rsync\" as documented or shut it down as a primary.\n");
-					else
-						pg_fatal("The target cluster was shut down while in recovery mode.  To upgrade, use \"rsync\" as documented or shut it down as a primary.\n");
-				}
-				else if (strcmp(p, "shut down\n") != 0)
-				{
-					if (cluster == &old_cluster)
-						pg_fatal("The source cluster was not shut down cleanly.\n");
-					else
-						pg_fatal("The target cluster was not shut down cleanly.\n");
-				}
-				got_cluster_state = true;
-			}
-		}
-
-		pclose(output);
-
-		if (!got_cluster_state)
-		{
-			if (cluster == &old_cluster)
-				pg_fatal("The source cluster lacks cluster state information:\n");
-			else
-				pg_fatal("The target cluster lacks cluster state information:\n");
-		}
-	}
-
 	snprintf(cmd, sizeof(cmd), "\"%s/%s \"%s\"",
 			 cluster->bindir,
 			 live_check ? "pg_controldata\"" : "pg_resetxlog\" -n",
@@ -186,8 +119,8 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	fflush(stderr);
 
 	if ((output = popen(cmd, "r")) == NULL)
-		pg_fatal("Could not get control data using %s: %s\n",
-				 cmd, getErrorText());
+		pg_fatal("could not get control data using %s: %s\n",
+				 cmd, strerror(errno));
 
 	/* Only in <= 9.2 */
 	if (GET_MAJOR_VERSION(cluster->major_version) <= 902)
@@ -264,11 +197,23 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 			p++;				/* remove ':' char */
 			cluster->controldata.chkpnt_nxtepoch = str2uint(p);
 
-			p = strchr(p, '/');
+			/*
+			 * Delimiter changed from '/' to ':' in 9.6.  We don't test for
+			 * the catalog version of the change because the catalog version
+			 * is pulled from pg_controldata too, and it isn't worth adding an
+			 * order dependency for this --- we just check the string.
+			 */
+			if (strchr(p, '/') != NULL)
+				p = strchr(p, '/');
+			else if (GET_MAJOR_VERSION(cluster->major_version) >= 906)
+				p = strchr(p, ':');
+			else
+				p = NULL;
+
 			if (p == NULL || strlen(p) <= 1)
 				pg_fatal("%d: controldata retrieval problem\n", __LINE__);
 
-			p++;				/* remove '/' char */
+			p++;				/* remove '/' or ':' char */
 			cluster->controldata.chkpnt_nxtxid = str2uint(p);
 			got_xid = true;
 		}
