@@ -1,7 +1,7 @@
 /* src/interfaces/ecpg/preproc/ecpg.c */
 
 /* Main for ecpg, the PostgreSQL embedded SQL precompiler. */
-/* Copyright (c) 1996-2015, PostgreSQL Global Development Group */
+/* Copyright (c) 1996-2016, PostgreSQL Global Development Group */
 
 #include "postgres_fe.h"
 
@@ -327,7 +327,6 @@ main(int argc, char *const argv[])
 						fprintf(stderr, _("%s: could not open file \"%s\": %s\n"),
 								progname, output_filename, strerror(errno));
 						free(output_filename);
-						output_filename = NULL;
 						free(input_filename);
 						continue;
 					}
@@ -475,10 +474,8 @@ main(int argc, char *const argv[])
 				}
 			}
 
-			if (output_filename && out_option == 0) {
+			if (output_filename && out_option == 0)
 				free(output_filename);
-				output_filename = NULL;
-			}
 
 			free(input_filename);
 		}
