@@ -108,7 +108,7 @@ int main()
 
 	ECPGdebug(1,stderr);
 
-	{ ECPGconnect(__LINE__, 0, "regress1" , NULL, NULL , NULL, 0); }
+	{ ECPGconnect(__LINE__, 0, "ecpg1_regression" , NULL, NULL , NULL, 0); }
 #line 15 "indicators.pgc"
 
 	{ ECPGsetcommit(__LINE__, "off", NULL);}
